'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ImageCell = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactPortal = require('react-portal');

var _reactPortal2 = _interopRequireDefault(_reactPortal);

var _constants = require('./constants');

var Constants = _interopRequireWildcard(_constants);

var _brokenImagePlaceholder = require('../assets/broken-image-placeholder.png');

var _brokenImagePlaceholder2 = _interopRequireDefault(_brokenImagePlaceholder);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ImageCell = exports.ImageCell = function (_React$Component) {
  _inherits(ImageCell, _React$Component);

  function ImageCell(props) {
    _classCallCheck(this, ImageCell);

    var _this = _possibleConstructorReturn(this, (ImageCell.__proto__ || Object.getPrototypeOf(ImageCell)).call(this, props));

    _this.handleMouseEnter = _this.handleMouseEnter.bind(_this);
    _this.handleMouseLeave = _this.handleMouseLeave.bind(_this);

    _this.state = {
      showPopover: false,
      imageErrored: false
    };
    return _this;
  }

  _createClass(ImageCell, [{
    key: 'handleMouseEnter',
    value: function handleMouseEnter(e) {
      var clientRect = e.target.getBoundingClientRect();

      this.setState({
        showPopover: true,
        popoverTop: (clientRect.top + clientRect.bottom) / 2,
        popoverLeft: clientRect.left + this.props.width - 20
      });
    }
  }, {
    key: 'handleMouseLeave',
    value: function handleMouseLeave() {
      this.setState({
        showPopover: false
      });
    }
  }, {
    key: 'handleImageLoaded',
    value: function handleImageLoaded() {
      // Treat 5x5 pixels as less as visually meaningless
      // load about:blank to show the alt text instead
      if (this.refs.img.width * this.refs.img.height <= 25) {
        this.refs.img.src = 'about:blank';
      }
    }
  }, {
    key: 'handleImageError',
    value: function handleImageError() {
      this.setState({ imageErrored: true });
    }
  }, {
    key: 'render',
    value: function render() {
      var cellData = this.props.cellData;
      var imageUrl = void 0,
          href = void 0,
          alt = void 0;

      if (cellData.main && _typeof(cellData.main) === 'object') {
        imageUrl = cellData.main.src;
        href = cellData.main.href || imageUrl;
        alt = cellData.alt || cellData.main.alt;
      } else {
        imageUrl = cellData.main;
        href = cellData.main;
        alt = cellData.alt || cellData.main;
      }

      if (!imageUrl) {
        return _react2.default.createElement('span', null);
      }

      if (this.props.mixedContentImage) {
        imageUrl = this.props.mixedContentImage(imageUrl, imageUrl);
      }

      return _react2.default.createElement(
        'a',
        {
          href: href,
          title: alt,
          target: '_blank',
          style: {
            display: 'inline-block'
          },
          onMouseEnter: this.handleMouseEnter,
          onMouseLeave: this.handleMouseLeave
        },
        _react2.default.createElement('img', {
          ref: 'img',
          className: 'example-image',
          src: this.state.imageErrored ? _brokenImagePlaceholder2.default : imageUrl,
          style: {
            maxHeight: Constants.ROW_HEIGHT - 20,
            maxWidth: this.props.width - 20,
            minHeight: 5,
            minWidth: 5,
            marginTop: '-4px'
          },
          title: alt,
          alt: alt ? alt : 'Image',
          onLoad: this.handleImageLoaded.bind(this),
          onError: this.handleImageError.bind(this)
        }),
        _react2.default.createElement(
          _reactPortal2.default,
          { isOpened: !!(imageUrl && this.state.showPopover) },
          _react2.default.createElement(
            'div',
            { className: 'popover fade right in', style: {
                display: 'block',
                position: 'absolute',
                top: this.state.popoverTop,
                left: this.state.popoverLeft,
                transform: 'translateY(-50%)',
                backgroundColor: '#fff'
              } },
            _react2.default.createElement('div', { className: 'arrow', style: { top: '50%' } }),
            _react2.default.createElement(
              'div',
              { className: 'popover-content' },
              _react2.default.createElement('img', {
                style: { width: '100%' },
                src: this.state.imageErrored ? _brokenImagePlaceholder2.default : imageUrl,
                title: this.state.imageErrored ? 'Image not found' : alt,
                alt: this.state.imageErrored ? 'Image not found' : alt
              }),
              _react2.default.createElement(
                'span',
                null,
                alt
              )
            )
          )
        )
      );
    }
  }]);

  return ImageCell;
}(_react2.default.Component);

ImageCell.propTypes = {
  cellData: _react2.default.PropTypes.object.isRequired,
  width: _react2.default.PropTypes.number.isRequired,
  mixedContentImage: _react2.default.PropTypes.func
};